@@ -28,20 +28,22 @@ type serviceServingCertUpdateController struct {
 	serviceLister listers.ServiceLister
 	secretLister  listers.SecretLister
 
-	ca        *crypto.CA
-	dnsSuffix string
+	ca                 *crypto.CA
+	intermediateCACert *x509.Certificate
+	dnsSuffix          string
 	// minTimeLeftForCert is how much time is remaining for the serving cert before regenerating it.
 	minTimeLeftForCert time.Duration
 }
 
-func NewServiceServingCertUpdateController(services informers.ServiceInformer, secrets informers.SecretInformer, secretClient kcoreclient.SecretsGetter, ca *crypto.CA, dnsSuffix string) controller.Runner {
+func NewServiceServingCertUpdateController(services informers.ServiceInformer, secrets informers.SecretInformer, secretClient kcoreclient.SecretsGetter, ca *crypto.CA, intermediateCACert *x509.Certificate, dnsSuffix string) controller.Runner {
 	sc := &serviceServingCertUpdateController{
 		secretClient:  secretClient,
 		serviceLister: services.Lister(),
 		secretLister:  secrets.Lister(),
 
-		ca:        ca,
-		dnsSuffix: dnsSuffix,
+		ca:                 ca,
+		intermediateCACert: intermediateCACert,
+		dnsSuffix:          dnsSuffix,
 		// TODO base the expiry time on a percentage of the time for the lifespan of the cert
 		minTimeLeftForCert: 1 * time.Hour,
 	}
@@ -87,7 +89,7 @@ func (sc *serviceServingCertUpdateController) Sync(obj metav1.Object) error {
 
 	if sc.requiresRegeneration(service, sharedSecret, sc.minTimeLeftForCert) {
 		// Regenerate the secret
-		if err := toRequiredSecret(sc.dnsSuffix, sc.ca, service, secretCopy); err != nil {
+		if err := toRequiredSecret(sc.dnsSuffix, sc.ca, sc.intermediateCACert, service, secretCopy); err != nil {
 			return err
 		}
 		_, err := sc.secretClient.Secrets(secretCopy.Namespace).Update(secretCopy)
@@ -180,7 +182,7 @@ func (sc *serviceServingCertUpdateController) ensureSecretData(service *v1.Servi
 	} else {
 		// if required tlscertkey,tlsprivatekey fields missing, replace with valid secret
 		// Regenerate the secret
-		if err := toRequiredSecret(sc.dnsSuffix, sc.ca, service, secretCopy); err != nil {
+		if err := toRequiredSecret(sc.dnsSuffix, sc.ca, sc.intermediateCACert, service, secretCopy); err != nil {
 			return update, err
 		}
 		return true, nil
@@ -192,7 +194,7 @@ func (sc *serviceServingCertUpdateController) ensureSecretData(service *v1.Servi
 		// Regenerate the secret
 		klog.Infof("Error decoding cert bytes %s from secret: %s namespace: %s, replacing cert", v1.TLSCertKey, secretCopy.Name, secretCopy.Namespace)
 		// Regenerate the secret
-		if err := toRequiredSecret(sc.dnsSuffix, sc.ca, service, secretCopy); err != nil {
+		if err := toRequiredSecret(sc.dnsSuffix, sc.ca, sc.intermediateCACert, service, secretCopy); err != nil {
 			return update, err
 		}
 		return true, nil
@@ -201,7 +203,7 @@ func (sc *serviceServingCertUpdateController) ensureSecretData(service *v1.Servi
 	if err != nil {
 		klog.Infof("Error parsing %s from secret: %s namespace: %s, replacing cert", v1.TLSCertKey, secretCopy.Name, secretCopy.Namespace)
 		// Regenerate the secret
-		if err := toRequiredSecret(sc.dnsSuffix, sc.ca, service, secretCopy); err != nil {
+		if err := toRequiredSecret(sc.dnsSuffix, sc.ca, sc.intermediateCACert, service, secretCopy); err != nil {
 			return update, err
 		}
 		return true, nil
